<?php
/**
 * @file
 * All the node functions being used for the webtexttool module.
 */

/**
 * Implements hook_form_FORM_ID_alter().
 */
function webtexttool_form_node_form_alter(&$form, &$form_state, $form_id) {
  $node = $form['#node'];

  // We only show the tool when user has access
  // and it is enabled on node type form.
  if (user_access('webtexttool_use') && variable_get('webtexttool_enabled_' . $node->type, 0) == 1) {
    $form['#attached']['css'][] = drupal_get_path('module', 'webtexttool') . '/css/webtexttool.css';
    drupal_add_library('system','ui.tabs');
    $form['#attached']['js'][] = drupal_get_path('module', 'webtexttool') . '/js/webtexttool.js';

    $form['#rebuild'] = TRUE;

    $form['webtexttool'] = array(
      '#type' => 'container',
      '#title' => t('Webtexttool'),
      '#weight' => 35,
      '#attributes' => array(
        'id' => 'webtexttool-analyse',
        'class' => array('webtexttool-form'),
      ),
    );

    $form['webtexttool']['logo'] = array(
      'logo' => array(
        '#theme' => 'link',
        '#options' => array(
          'html' => TRUE,
          'attributes' => array(
            'target' => '_blank',
          )
        ),
        '#text' => '<img class="logo" src="' . base_path() . drupal_get_path('module', 'webtexttool') . '/img/wtt_logo.png">',
        '#path' => 'https://app.webtexttool.com'
      )
    );

    $can_login = FALSE;
    $is_authenticated = _webtexttool_is_authenticated();
    if (!$is_authenticated) {
      $login = _webtexttool_login();
      if ($login) {
        $can_login = TRUE;
      }
    }
    else {
      $can_login = TRUE;
    }


    if (!$can_login) {
      $form['webtexttool']['message'] = array(
        '#type' => 'container',
        '#attributes' => array(
          'class' => array('info')
        ),
        'text' => array(
          '#markup' => t('Unable to login with the credentials you filled in at !url', array('!url' => l(t('webtexttool acount form'), 'admin/config/services/webtexttool')))
        ),
      );
    }
    else {
      $form['webtexttool']['tabs_wrapper'] = array(
        '#type' => 'container',
        '#attributes' => array(
          'class' => array(),
          'id' => 'webtexttool-tabs-wrapper'
        ),
        'list' => array(
          '#theme' => 'item_list',
          '#items' => array(),
          '#attributes' => array(
            'class' => array(),
          )
        )
      );

      $tab_info = array(
        'seo' => t('SEO'),
        'content_quality' => t('Content quality'),
      );

      foreach ($tab_info as $tab => $tab_label) {
        $form['webtexttool']['tabs_wrapper'][$tab] = array(
          '#type' => 'container',
          '#attributes' => array(
            'id' => 'webtexttool-' . $tab
          )
        );

        $form['webtexttool']['tabs_wrapper'][$tab]['inner'] = array(
          '#type' => 'container',
          '#attributes' => array(
            'id' => 'webtexttool-' . $tab . '-inner'
          )
        );

        $form['webtexttool']['tabs_wrapper']['list']['#items'][$tab] = array(
          '<a href="#webtexttool-' . $tab . '">' . $tab_label . '</a>'
        );
      }

      $form['webtexttool']['tabs_wrapper']['seo']['inner']['analyse_seo'] = array(
        '#type' => 'button',
        '#value' => t('Get SEO suggestions'),
        '#attributes' => array('class' => array('wtt-start-optimizing-seo')),
        '#ajax' => array(
          'callback' => 'webtexttool_analyse_seo_form_node_callback',
          'wrapper' => 'webtexttool-seo-inner',
          'event' => 'click',
          'method' => 'replace',
        ),
      );

      $form['webtexttool']['tabs_wrapper']['content_quality']['inner']['analyse_content_quality'] = array(
        '#type' => 'button',
        '#value' => t('Analyse content quality'),
        '#title' => t('Get keyword suggestion'),
        '#prefix' => '<div class="wtt-analyse-content-quality-wrapper">',
        '#suffix' => '</div>',
        '#ajax' => array(
          'callback' => 'webtexttool_analyse_content_quality_form_node_callback',
          'wrapper' => 'webtexttool-content_quality-inner',
          'event' => 'click',
          'method' => 'replace',
        ),
      );

      $checked = '';

      if (isset($node->webtexttool_keywords) && $node->webtexttool_keywords) {
        $checked = 'checked="checked"';
      }


      $form['webtexttool']['tabs_wrapper']['seo']['webtexttool_keywords'] = array(
        '#prefix' => '<div class="wtt-form-element wtt-form-element-keywords"><input type="checkbox" ' . $checked . ' id="wtt-sug-label-webtexttool-keywords"/><label for="wtt-sug-label-webtexttool-keywords">' . t('Keyword') . '</label>',
        '#suffix' => '</div>',
        '#type' => 'textfield',
        '#weight' => -1,
        '#default_value' => isset($node->webtexttool_keywords) ? $node->webtexttool_keywords : '',
        '#description' => t('Enter the keywords which should be analyzed.'),
        '#maxlength' => 512,
      );

      $form['webtexttool']['tabs_wrapper']['seo']['suggestions'] = array(
        '#type' => 'container',
      );

      $form['webtexttool']['tabs_wrapper']['seo']['webtexttool_language'] = array(
        '#prefix' => '<div class="wtt-form-element"><input type="checkbox" checked="checked" id="wtt-sug-label-webtexttool-language"/><label for="wtt-sug-label-webtexttool-language">' . t('Keyword suggestions language') . '</label>',
        '#suffix' => '</div>',
        '#type' => 'select',
        '#options' => webtexttool_getkeywordsources(),
        '#description' => t('Based on the keyword we can lookup synonyms that can be added in the text to score even higher in search engines. This languague will be used to lookup synonyms.'),
        '#default_value' => isset($node->webtexttool_language) ? $node->webtexttool_language : '',
      );

      $form['webtexttool']['tabs_wrapper']['seo']['get_keyword_suggestion'] = array(
        '#type' => 'button',
        '#value' => t('Get keyword suggestion'),
        '#title' => t('Get keyword suggestion'),
        '#ajax' => array(
          'callback' => 'webtexttool_suggestion_form_node_callback',
          'wrapper' => 'webtexttool-keyword-suggestion',
          'effect' => 'fade',
          'event' => 'click',
        ),
      );

      $form['webtexttool']['tabs_wrapper']['seo']['keyword_container'] = array(
        '#prefix' => '<div id="webtexttool-keyword-suggestion">',
        '#suffix' => '</div>',
      );
    }

    $form_state = _webtexttool_update_form_state_values($form_state, $node);
    $node = node_form_submit_build_node($form, $form_state);
    $node_markup = webtexttool_get_formstate_node_markup($form, $form_state);

    // Check response code.
    if (isset($node_markup) && $node_markup) {
      $langcode = $node->language == LANGUAGE_NONE ? variable_get('webtexttool_language', 'en') : $node->language;

      if (isset($form_state["values"]["webtexttool_content_quality_settings"])) {
        $node->webtexttool_content_quality_settings = $form_state["values"]["webtexttool_content_quality_settings"];
      }

      if (!isset($node->webtexttool_content_quality_settings)) {
        $node->webtexttool_content_quality_settings = array();
      }

      $form_state['node'] = $node;
      $response = webtexttool_analyse_content_quality($node_markup, $node->webtexttool_content_quality_settings, $langcode);

      $form['webtexttool']['tabs_wrapper']['content_quality']['response'] = array(
        '#type' => 'hidden',
        '#value' => json_encode($response),
      );

      if (is_array($response) && count($response)) {
        if (isset($response['stats'])) {

          $markup = FALSE;
          foreach ($response['stats'] as $stat_label => $stat_value) {

            if ($stat_value) {
              if (!$markup) {
                $markup = '<div class="stat"><strong>' . $stat_label . ':</strong><span>' . $stat_value . '</span></div>';
              }
              else {
                $markup .= '<div class="stat"><strong>' . $stat_label . ':</strong><span>' . $stat_value . '</span></div>';
              }
            }
          }

          $form['webtexttool']['tabs_wrapper']['content_quality']['inner'][$stat_label] = array(
            '#type' => 'markup',
            '#markup' => $markup,
            '#prefix' => '<div class="statistics">',
            '#suffix' => '</div>'
          );
        }

        if (isset($response['suggestions'])) {

          $counter = 0;

          foreach ($response['suggestions'] as $delta => $suggestion) {
            $suggestion_name = $suggestion['Tag'];

            $form['webtexttool']['tabs_wrapper']['content_quality']['inner']['webtexttool_content_quality_settings']['toggle_' . $suggestion_name] = array(
              '#weight' => 10 + $counter,
              '#type' => 'checkbox',
              '#name' => 'toggle_' . $suggestion['Tag'],
              '#prefix' => '<div class="wtt-toggle-switch">',
              '#suffix' => '</div>',
              '#attributes' => array('class' => array('wtt-toggle-form-item')),
              '#title' => ' ',
              '#default_value' => isset($node->webtexttool_content_quality_settings['toggle_' . $suggestion_name]) ? $node->webtexttool_content_quality_settings['toggle_' . $suggestion_name] : 0,
            );

            $counter++;

            $setting = array();

            switch ($suggestion['Tag']) {

              // Build the readability settings
              case 'Readability';

                $setting['readability_setting'] = array(
                  '#type' => 'radios',
                  '#options' => array(
                    1 => t('Elementary'),
                    2 => t('Highschool'),
                    3 => t('University'),
                  ),
                  '#default_value' => isset($node->webtexttool_content_quality_settings['Readability']['readability_setting']) ? $node->webtexttool_content_quality_settings['Readability']['readability_setting'] : 0,
                );

                break;

              // Build the gender settings
              case 'Gender';

                $setting['gender_setting'] = array(
                  '#type' => 'radios',
                  '#options' => array(
                    'f' => t('Female'),
                    'n' => t('Neutral'),
                    'm' => t('Male'),
                  ),
                  '#default_value' => isset($node->webtexttool_content_quality_settings['Gender']['gender_setting']) ? $node->webtexttool_content_quality_settings['Gender']['gender_setting'] : '',
                );

                break;

              // Build the sentiment settings
              case 'Sentiment';

                $setting['sentiment_setting'] = array(
                  '#type' => 'radios',
                  '#options' => array(
                    'negative' => t('Negative'),
                    'neutral' => t('Neutral'),
                    'positive' => t('Positive'),
                  ),
                  '#default_value' => isset($node->webtexttool_content_quality_settings['Sentiment']['sentiment_setting']) ? $node->webtexttool_content_quality_settings['Sentiment']['sentiment_setting'] : '',
                );

                break;
            }

            // Build the suggestion render array elements, and add the settings to it.
            $form['webtexttool']['tabs_wrapper']['content_quality']['inner']['webtexttool_content_quality_settings'][$suggestion_name] = array(
                '#weight' => 10 + $counter,
                '#theme' => 'webtexttool_suggestion',
                '#suggestion' => $suggestion,
              ) + $setting;

            $counter++;
          }
        }
      }
    }
  }
}

/**
 * AJAX callback for the webtexttool button.
 */
function webtexttool_analyse_content_quality_form_node_callback($form, $form_state) {
  webtexttool_update_changed_state($form_state);
  $node = node_form_submit_build_node($form, $form_state);
  $node_markup = webtexttool_get_formstate_node_markup($form, $form_state);

  // Check response code.
  if (isset($node_markup) && $node_markup) {
    $langcode = $node->language == LANGUAGE_NONE ? variable_get('webtexttool_language', 'en') : $node->language;

    if (isset($form_state["values"]["webtexttool_content_quality_settings"])) {
      $node->webtexttool_content_quality_settings = $form_state["values"]["webtexttool_content_quality_settings"];
    }

    if (!isset($node->webtexttool_content_quality_settings)) {
      $node->webtexttool_content_quality_settings = array();
    }

    // Inject the updated node into the form state.
    $form_state['node'] = $node;
    $response = webtexttool_analyse_content_quality($node_markup, $node->webtexttool_content_quality_settings, $langcode);

    $form['webtexttool']['tabs_wrapper']['content_quality']['response']['#value'] = json_encode($response);

    if ($response) {
      if (isset($response['stats'])) {

        $markup = '';
        foreach ($response['stats'] as $stat_label => $stat_value) {
          $markup .= '<div class="stat"><strong>' . $stat_label . ':</strong><span>' . $stat_value . '</span></div>';
        }

        $form['webtexttool']['tabs_wrapper']['content_quality']['inner'][$stat_label]['#markup'] = $markup;
      }

      if (isset($response['suggestions'])) {
        foreach ($response['suggestions'] as $suggestion) {
          $suggestion_name = $suggestion['Tag'];

          // Update the switch
          $form['webtexttool']['tabs_wrapper']['content_quality']['inner']['webtexttool_content_quality_settings']['toggle_' . $suggestion_name]['#default_value'] = isset($node->webtexttool_content_quality_settings['toggle_' . $suggestion_name]) ? $node->webtexttool_content_quality_settings['toggle_' . $suggestion_name] : 0;

          // Update the suggestions.
          $form['webtexttool']['tabs_wrapper']['content_quality']['inner']['webtexttool_content_quality_settings'][$suggestion_name]['#suggestion'] = $suggestion;

          $setting = array();

          switch ($suggestion['Tag']) {

            case 'Readability';
              $setting['readability_setting']['#default_value'] = isset($node->webtexttool_content_quality_settings['Readability']['readability_setting']) ? $node->webtexttool_content_quality_settings['Readability']['readability_setting'] : 0;
              break;

            case 'Gender';
              $setting['gender_setting']['#default_value'] = isset($node->webtexttool_content_quality_settings['Gender']['gender_setting']) ? $node->webtexttool_content_quality_settings['Gender']['gender_setting'] : '';
              break;

            case 'Sentiment';
              $setting['sentiment_setting']['#default_value'] = isset($node->webtexttool_content_quality_settings['Sentiment']['sentiment_setting']) ? $node->webtexttool_content_quality_settings['Sentiment']['sentiment_setting'] : '';
              break;
          }

          $form['webtexttool']['tabs_wrapper']['content_quality']['inner']['webtexttool_content_quality_settings'][$suggestion_name] += $setting;
        }
      }
    }
  }

  return $form['webtexttool']['tabs_wrapper']['content_quality']['inner'];
}

/**
 * Updates the form state values with the input values or the node values as fallback.
 */
function _webtexttool_update_form_state_values($form_state, $node) {
  if (!isset($node->webtexttool_content_quality_settings["toggle_Readability"])) {
    $node->webtexttool_content_quality_settings["toggle_Readability"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']["toggle_Readability"] = array_key_exists('toggle_Readability', $form_state["input"]) ? (int) $form_state["input"]["toggle_Readability"] : $node->webtexttool_content_quality_settings["toggle_Readability"];

  if (!isset($node->webtexttool_content_quality_settings["Readability"]["readability_setting"])) {
    $node->webtexttool_content_quality_settings["Readability"]["readability_setting"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']['Readability']["readability_setting"] = array_key_exists('readability_setting', $form_state["input"]) ? (int) $form_state["input"]["readability_setting"] : $node->webtexttool_content_quality_settings["Readability"]["readability_setting"];

  if (!isset($node->webtexttool_content_quality_settings["toggle_Bulletpoints"])) {
    $node->webtexttool_content_quality_settings["toggle_Bulletpoints"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']["toggle_Bulletpoints"] = array_key_exists('toggle_Bulletpoints', $form_state["input"]) ? (int) $form_state["input"]["toggle_Bulletpoints"] : $node->webtexttool_content_quality_settings["toggle_Bulletpoints"];

  if (!isset($node->webtexttool_content_quality_settings["toggle_Adjectives"])) {
    $node->webtexttool_content_quality_settings["toggle_Adjectives"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']["toggle_Adjectives"] = array_key_exists('toggle_Adjectives', $form_state["input"]) ? (int) $form_state["input"]["toggle_Adjectives"] : $node->webtexttool_content_quality_settings["toggle_Adjectives"];

  if (!isset($node->webtexttool_content_quality_settings["toggle_Whitespaces"])) {
    $node->webtexttool_content_quality_settings["toggle_Whitespaces"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']["toggle_Whitespaces"] = array_key_exists('toggle_Whitespaces', $form_state["input"]) ? (int) $form_state["input"]["toggle_Whitespaces"] : $node->webtexttool_content_quality_settings["toggle_Whitespaces"];

  if (!isset($node->webtexttool_content_quality_settings["toggle_Gender"])) {
    $node->webtexttool_content_quality_settings["toggle_Gender"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']["toggle_Gender"] = array_key_exists('toggle_Gender', $form_state["input"]) ? (int) $form_state["input"]["toggle_Gender"] : $node->webtexttool_content_quality_settings["toggle_Gender"];

  if (!isset($node->webtexttool_content_quality_settings["Gender"]["gender_setting"])) {
    $node->webtexttool_content_quality_settings["Gender"]["gender_setting"] = 'n';
  }
  $form_state["values"]['webtexttool_content_quality_settings']['Gender']["gender_setting"] = array_key_exists('gender_setting', $form_state["input"]) ? $form_state["input"]["gender_setting"] : $node->webtexttool_content_quality_settings["Gender"]["gender_setting"];

  if (!isset($node->webtexttool_content_quality_settings["toggle_Sentiment"])) {
    $node->webtexttool_content_quality_settings["toggle_Sentiment"] = 1;
  }
  $form_state["values"]['webtexttool_content_quality_settings']["toggle_Sentiment"] = array_key_exists('toggle_Sentiment', $form_state["input"]) ? (int) $form_state["input"]["toggle_Sentiment"] : $node->webtexttool_content_quality_settings["toggle_Sentiment"];

  if (!isset($node->webtexttool_content_quality_settings["Sentiment"]["sentiment_setting"])) {
    $node->webtexttool_content_quality_settings["Sentiment"]["sentiment_setting"] = 'neutral';
  }
  $form_state["values"]['webtexttool_content_quality_settings']['Sentiment']["sentiment_setting"] = array_key_exists('sentiment_setting', $form_state["input"]) ? $form_state["input"]["sentiment_setting"] : $node->webtexttool_content_quality_settings["Sentiment"]["sentiment_setting"];

  webtexttool_update_changed_state($form_state);

  return $form_state;
}

/**
 * Updates the changed state of the node for calling the wtt while adding a new node.
 */
function webtexttool_update_changed_state(&$form_state) {
  if (!isset($form_state["node"]->changed) || $form_state["node"]->changed == '') {
    $form_state["node"]->changed = $form_state["node"]->created;
  }
}

/**
 * Returns changed part of the form.
 *
 * @return array
 *   Form API array.
 *
 * @see ajax_example_form_node_form_alter()
 */
function webtexttool_suggestion_form_node_callback($form, &$form_state) {
  $values = $form_state['values'];
  $keywords_value = $values['webtexttool_keywords'];
  $database = $values['webtexttool_language'];
  $keywords = explode(',', $keywords_value);
  global $language;

  $header = array(
    t("Keyword"),
    t("Search volume"),
    t("Competition"),
    t("Overall"),
  );

  $rows = array();

  foreach ($keywords as $keyword) {
    $response = webtexttool_searchkeyword($keyword, $database, $language->language);
    if ($response && is_array($response)) {
      foreach ($response as $suggested_keyword) {
        $keyword = $suggested_keyword['Keyword'];
        $volumescore = $suggested_keyword['VolumeScore'];
        $competitionscore = $suggested_keyword['CompetitionScore'];
        $overallscore = $suggested_keyword['OverallScore'];
        $volumeindicator = webtexttool_color_indicator(webtexttool_score_to_class($volumescore));
        $competitionindicator = webtexttool_color_indicator(webtexttool_score_to_class($competitionscore));
        $overallindicator  = webtexttool_color_indicator(webtexttool_overall_score_to_class($overallscore));

        $rows[] = array(
          $keyword,
          $volumeindicator . webtexttool_voilume_score_to_text($volumescore),
          $competitionindicator . webtexttool_competition_score_to_text($competitionscore),
          $overallindicator . webtexttool_overall_to_text($overallscore),
        );
      }
    }
  }

  $output = theme('table', array('header' => $header, 'rows' => $rows));

  $commands[] = ajax_command_html('#webtexttool-keyword-suggestion', $output);
  return array('#type' => 'ajax', '#commands' => $commands);
}

function webtexttool_get_formstate_node_markup($form, $form_state) {
  $node = node_form_submit_build_node($form, $form_state);

  // Check if we have a valid nid.
  $node->nid = !empty($node->nid) ? $node->nid : 0;
  if (module_exists('metatag')) {

    // First we need to get the metatags defaults.
    $default_metatags = $form['metatags'][$node->language]['#metatag_defaults'];

    // Now we merge the defaults with the changes in the form.
    foreach ($default_metatags as $key => $metatag) {

      if (!isset($node->metatags[$node->language][$key])) {
        // Set a default value for a metatag das is using the default.
        $node->metatags[$node->language][$key]['value'] = $metatag['value'];

      }
    }
    // Replace all tokens wit the correct value.
    foreach ($node->metatags[$node->language] as $key => $metatag) {
      $node->metatags[$node->language][$key]['value'] = token_replace($node->metatags[$node->language][$key]['value'], array('node' => $node));
    }
  }

  // Now we are going to post to the new menu item.
  $token = drupal_random_key();
  $data = 'token=' . $token . '&node=' . drupal_json_encode($node);
  variable_set('webtexttool_token', $token);

  $options = array(
    'method' => 'POST',
    'data' => $data ,
    'headers' => array(
      'Content-Type' => 'application/x-www-form-urlencoded',
    ),
  );

  // Adding authorisation to the call when the drupal site is.
  if (variable_get('webtexttool_authenticate', 0) == 1 && variable_get('webtexttool_authenticate_credentials', '') != '') {
    $options['headers']['Authorization'] = 'Basic ' . base64_encode(variable_get('webtexttool_authenticate_credentials', ''));
  }

  $url = url('webtexttool/view', array('absolute' => TRUE));

  if(module_exists('chr')){
    $request = chr_curl_http_request($url, $options);
  }
  else{
    $request = drupal_http_request($url, $options);
  }

  // Check response code.
  if ($request->code == 200 && $request->status_message == 'OK') {
    return $request->data;
  }
}

/**
 * AJAX callback for the webtexttool button.
 */
function webtexttool_analyse_seo_form_node_callback($form, &$form_state) {
  $node = node_form_submit_build_node($form, $form_state);

  if (!isset($node->webtexttool_keywords) || $node->webtexttool_keywords == '') {
    $form['webtexttool']['message'] = array(
      '#weight' => -1,
      '#type' => 'container',
      '#attributes' => array(
        'class' => array('info')
      ),
      'text' => array(
        '#markup' => t('Please, enter a keyword in the Webtexttool-tab to get started.')
      ),
    );
  }

  else {
    $node_markup = webtexttool_get_formstate_node_markup($form, $form_state);

    // Check response code.
    if ($node_markup) {
      $langcode = $node->language == LANGUAGE_NONE ? variable_get('webtexttool_language', 'en') : $node->language;
      $response = webtexttool_analyse_seo($node_markup, $node->webtexttool_keywords, $langcode);
      if ($response) {
       $stat_items = array();

        foreach ($response['stats'] as $stat_label => $stat_value) {
          $stat_items[] = '<strong>' . $stat_label . ':</strong> ' . $stat_value;
        }

        foreach ($response['suggestions'] as $suggestion) {
          $form['webtexttool']['tabs_wrapper']['seo']['inner'][] = array(
            '#theme' => 'webtexttool_suggestion',
            '#suggestion' => $suggestion
          );
        }
      }
    }
    else {
      $output[] = array(
        '#type' => 'container',
        '#attributes' => array(
          'class' => array('info')
        ),
        '#markup' => t('Unable to load html of this node. Please check if an anonymous user can access a full rendered node of this type.')
      );
    }

  }

  return $form['webtexttool']['tabs_wrapper']['seo']['inner'];
}

/**
 * Implements hook_ajax_render_alter().
 */
function webtexttool_ajax_render_alter(&$commands) {
  $webtexttool_found = FALSE;

  foreach ($commands as $command) {
    if (isset($command['data']) && substr($command['data'], 0, 29) == '<div id="webtexttool-analyse"') {
      $webtexttool_found = TRUE;
    }
  }

  if ($webtexttool_found) {
    foreach ($commands as $delta => $command) {
      if (isset($command['data']) && substr($command['data'], 0, 20) == '<div class="messages') {
        unset($commands[$delta]);
      }
    }
  }
}

/**
 * Implements hook_node_load().
 */
function webtexttool_node_load($nodes, $types) {
  $vids = array();
  foreach ($nodes as $node) {
    $vids[$node->vid] = $node->vid;
  }
  $result = db_query('SELECT nid, keywords, lang, content_quality_settings FROM {webtexttool} WHERE vid IN(:vids)', array(':vids' => array_keys($vids)));

  foreach ($result as $record) {
    $nodes[$record->nid]->webtexttool_keywords = $record->keywords;
    $nodes[$record->nid]->webtexttool_language = $record->lang;
    $nodes[$record->nid]->webtexttool_content_quality_settings = drupal_json_decode($record->content_quality_settings, TRUE);
  }
}

/**
 * Implements hook_node_insert().
 */
function webtexttool_node_insert($node) {
  // Only insert into database if we need to exclude the node.
  if (!empty($node->webtexttool_keywords) || !empty($node->webtexttool_language) || !empty($node->webtexttool_content_quality_settings)) {
    db_insert('webtexttool')
      ->fields(array(
        'nid' => $node->nid,
        'vid' => $node->vid,
        'keywords' => isset($node->webtexttool_keywords) ? $node->webtexttool_keywords : '',
        'lang' => isset($node->webtexttool_language) ? $node->webtexttool_language : '',
        'content_quality_settings' => isset($node->webtexttool_content_quality_settings) ? json_encode($node->webtexttool_content_quality_settings) : '',
      ))
      ->execute();
  }
}

/**
 * Implements hook_node_update().
 */
function webtexttool_node_update($node) {
  // If the user created a new revision will we insert a new record in the db.
  // Else update database if we need to exclude the node.
  // Otherwise we delete the record from the database.
  if (!$node->is_new && !empty($node->revision) && $node->vid) {
    webtexttool_node_insert($node);
  }
  elseif (!empty($node->webtexttool_keywords) || !empty($node->webtexttool_language) || !empty($node->webtexttool_content_quality_settings)) {
    db_merge('webtexttool')
      ->key(array('vid' => $node->vid))
      ->fields(array(
        'nid' => $node->nid,
        'keywords' => $node->webtexttool_keywords ? $node->webtexttool_keywords : '',
        'lang' => $node->webtexttool_language ? $node->webtexttool_language : '',
        'content_quality_settings' => $node->webtexttool_content_quality_settings ? drupal_json_encode($node->webtexttool_content_quality_settings) : '',
      ))
      ->execute();
  }
  else {
    webtexttool_node_revision_delete($node);
  }
}

/**
 * Implements hook_node_delete().
 */
function webtexttool_node_delete($node) {
  db_delete('webtexttool')
    ->condition('nid', $node->nid)
    ->execute();
}

/**
 * Implements hook_node_revision_delete().
 */
function webtexttool_node_revision_delete($node) {
  db_delete('webtexttool')
    ->condition('vid', $node->vid)
    ->execute();
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function webtexttool_form_node_type_form_alter(&$form, &$form_state) {
  if (isset($form['type'])) {
    $form['webtexttool'] = array(
      '#type' => 'fieldset',
      '#title' => t('Webtexttool settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#group' => 'additional_settings',
      '#attributes' => array(
        'class' => array('webtexttool-node-type-settings-form'),
      ),
      '#attached' => array(
        'js' => array(drupal_get_path('module', 'webtexttool') . '/js/webtexttool_node_type.js'),
      ),
    );

    $form['webtexttool']['webtexttool_enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enabled'),
      '#default_value' => variable_get('webtexttool_enabled_' . $form['#node_type']->type, 0),
      '#description' => t('Enable the webtexttool for this contenttype.'),
    );
  }
}

/**
 * Function to render a node based on a post from the form.
 */
function webtexttool_view() {
  $current_token = variable_get('webtexttool_token', '');
  if (isset($_POST) && isset($_POST['token'])) {
    if ($_POST['token'] != $current_token) {
      drupal_not_found();
      drupal_exit();
    }
  }
  else {
    drupal_not_found();
    drupal_exit();
  }
  // First we need to get the post data.
  if (isset($_POST) && isset($_POST['node'])) {
    $node = (object) drupal_json_decode($_POST['node']);
  }
  else {
    drupal_not_found();
    drupal_exit();
  }

  // Getting globals.
  global $user;

  drupal_page_is_cacheable(FALSE);
  // Make a copy of the current user and set new current user as anonymous.
  $current_user = clone $user;
  $user = drupal_anonymous_user();

  // Re-initialize path.
  $_GET['q'] = 'node';
  if ($_GET['q'] == 'node' && !empty($node->nid)) {
    $_GET['q'] .= '/' . $node->nid;
  }

  // Render all metatags by hand.
  $head = '';
  if (isset($node->metatags) && isset($node->metatags[$node->language])) {
    foreach ($node->metatags[$node->language] as $meta_key => $meta_value) {
      if (isset($meta_value['value'])) {
        if ($meta_key == 'title') {
          $head .= '<title>' . strip_tags($meta_value['value']) . '</title>';
        }
        else {
          $head .= '<meta name="' . check_plain($meta_key) . '" content="' . strip_tags($meta_value['value']) . '" /> ';
        }
      }
    }
  }
  // Render the custom <meta name="url" content="">.
  $path = '';
  if (module_exists('path') && isset($node->path) && $node->path['alias'] != '') {
    $path = $node->path['alias'];
  }

  if (module_exists('pathauto') && isset($node->path) && $node->path['pathauto'] == 1) {
    $pattern = pathauto_pattern_load_by_entity('node', $node->type, $node->language);
    $path = token_replace($pattern, array('node' => $node));
  }

  if ($path != '') {
    $path = url($path);
    $head .= '<meta name="url" content="' . $path . '" /> ';
  }

  // Get a very plain version of the html.
  $page = '<html><head>' . $head;
  $page .= '</head><body>';

  // Render node.
  $node_view = node_view($node);
  $key = empty($node->nid) ? $node->nid : 0;
  $nodes['nodes'][$key] = $node_view;

  // Set page content, page title and render page.
  drupal_set_title(check_plain($node->title));
  drupal_set_page_content($node_view);
  $page .= drupal_render($node_view);
  $page .= '</body></html>';

  // Change back user object.
  $user = $current_user;
  print $page;
}
